<?php

namespace WSGateway;

/**
*	Simple REST to WebSocket API connector
*	
*	For PHP 5.3+, 7, 8
*/
class Rest2WebSocket {
	
	protected $serverKey = null;
	protected $accessKey = null;
	protected $accessToken = null;
	
	protected $response = null;
	protected $errors = null;
	
	protected $apiPath = null;
	
	/**
	 *	Constructor for Rest2WebSocket API connector
	 */
	public function __construct($serverKey = null, $accessKey = null, $accessToken = null, $apiPath = null) {
		$this->init($serverKey, $accessKey, $accessToken, $apiPath);
	}
	
	/**
	 *	Init instance with the server key, access key and token
	 *	
	 *	@param string $serverKey     WebSocket server key
	 *	@param string $accessKey     Access key to remote API
	 *	@param string $accessToken   Access token to remote API
	 *	@param string $apiPath       URL to remote API
	 *	
	 *	@return bool                 Return true if success or false if error
	 */
	public function init($serverKey = null, $accessKey = null, $accessToken = null, $apiPath = null) {
		$output = false;
		
		$this->reset();
		
		// if serverKye is config file URI
		if (
				is_string($serverKey) && !empty($serverKey) && is_null($accessKey) && is_null($accessToken)
				&& preg_match("/^([a-z0-9_\-\/\.]+)([a-z0-9_\-]+)\.php$/i", $serverKey) && file_exists($serverKey)
			) {
			$config = null;
			if (include($serverKey)) {
				if (isset($config['serverKey'])) $serverKey = $config['serverKey'];
				if (isset($config['accessKey'])) $accessKey = $config['accessKey'];
				if (isset($config['accessToken'])) $accessToken = $config['accessToken'];
				if (isset($config['apiPath'])) $apiPath = $config['apiPath'];
			}
			unset($config);
		}
		
		if (
				isset($serverKey) && is_string($serverKey) 
				&& isset($accessKey) && is_string($accessKey)
				&& isset($accessToken) && is_string($accessToken)
				&& preg_match("/^[a-z0-9]{5,32}$/i", $serverKey)
				&& preg_match("/^[a-z0-9]{5,32}$/i", $accessKey)
				&& preg_match("/^[a-z0-9][a-z0-9_#\~\-\&\$\%\^\*\!\?]{10,48}$/i", $accessToken)
				&& preg_match("/^https:\/\/[a-z][a-z0-9\.\-\/]+$/", $apiPath)
			) {
			$this->server = $serverKey;
			$this->accessKey = $accessKey;
			$this->accessToken = $accessToken;
			$this->apiPath = $apiPath;
			$output = true;
		}
		
		return($output);
	}
	
	/**
	 *	Reset instance properties
	 */
	public function reset() {
		$this->server = null;
		$this->accessKey = null;
		$this->accessToken = null;
		$this->apiPath = null;
		$this->response = null;
		$this->errors = null;
	}
	
	/**
	 *	Call remote API with the service name, action, and data as parameters
	 *	
	 *	@param string $service   Service name called on websocket server
	 *	@param string $action    Action called on remote websocket service
	 *	@param array  $data      Data array for send to remote service
	 *	
	 *	@return object           Return response data from remote API as array, null or false if error exists
	 */
	public function call($service = null, $action = null, $data = null) {
		//default return false
		$output = false;
				
		//set default error (data of response is set below)
		$this->response = false;
		$this->errors = [];
		
		if (
				isset($this->server) && isset($this->accessKey) && isset($this->accessToken)
				&& isset($service) && is_string($service)
				&& isset($action) && is_string($action)
				&& (is_null($data) || is_array($data) || is_object($data))
				&& preg_match("/^[a-z0-9]{5,32}$/i", $service)
				&& preg_match("/^([a-z0-9]+[\.]?)+$/i", $action)
			) {
			
			$curl = curl_init();
			if ($curl) {
				
				$httpHeaders = [
					'Authorization: Bearer '.base64_encode($this->accessToken)
				];
				
				$requestData = [
					"server" => $this->server		//websocket server key
					, "service" => $service			//short websocket service name (without svc/*)
					, "action" => $action			//action for WS API
				];									//append data of notification
				
				if (is_object($data)) {
					$data = (array) $data;
				}
				
				if (is_array($data)) {
					$requestData += $data;
				}
				
				$curlOptions = array(
					CURLOPT_URL => $this->apiPath.$service.'/~'.$this->accessKey
					, CURLOPT_POST => true
					, CURLOPT_RETURNTRANSFER => true
					, CURLOPT_HTTPHEADER => $httpHeaders
					, CURLOPT_POSTFIELDS => http_build_query($requestData)
				);
				
				//set curl options multiple
				curl_setopt_array($curl, $curlOptions);
				
				//exec url call and get response data
				$result = curl_exec($curl);
				
				//if curl error not exists
				if (curl_errno($curl) === 0) {
					
					//if data is JSON string
					if (
							is_string($result) && strlen($result) > 4
							&& strpos($result, "{") !== false 
							&& strpos($result, "}") !== false
						) {
						//parse JSON to array
						$this->response = json_decode($result, true);
						
						//if response is OK, return data
						if (isset($this->response['status']) && $this->response['status'] == 'OK') {
							$output = $this->response;
							
						//if response contains error
						} elseif (isset($this->response['error']) && !empty($this->response['error'])) {
							$this->errors[] = $this->response['error'];
							
						//else general error
						} else {
							$this->errors[] = 'Error API response.';
						}
						
					} else {
						$this->errors[] = 'CURL response is not valid.';
					}
					
				//error of curl execution
				} else { 
					$this->errors[] = curl_error($curl);
				} //end curl error
				
				unset($httpHeaders, $curlOptions, $requestData, $result);
				
			//else if curl is not initialized
			} else {
				$this->errors[] = 'Error of CURL initialization.';
			}
			
			curl_close($curl);
			unset($curl);
		
		//else if accss data is not correct or set
		} else {
			$this->errors[] = 'Error API access data.';
		}
		
		return($output);
	}
	
	/**
	 *	Get response data from remote API
	 *	
	 *	@return array    Return response data array, null or false
	 */
	public function getResponse() {
		return($this->response);
	}
	
	/**
	 *	Get CURL errors if exists
	 *	
	 *	@return array    Return list of CURL errors
	 */
	public function getErrors() {
		$output = null;
		if (is_array($this->errors) && !empty($this->errors)) {
			$output = $this->errors;
		}
		return($output);
	}
	
} //end class Rest2WebSocket

?>